
/* =========================================================================== *
|                                                                              |
|  Copyright (c) 2003, by members of University of Arizona Computer Vision     |
|  group (the authors) including                                               ||
|        Kobus Barnard.                                                        |
|        Prasad Gabbur.                                                        |
|  For use outside the University of Arizona Computer Vision group please      |
|  contact Kobus Barnard.                                                      |
|                                                                              |
* =========================================================================== */

#include "i/i_incl.h"
#include "i/i_float_io.h" 
#include "seg/seg_incl.h" 

/* -------------------------------------------------------------------------- */

int main(int argc, char** argv)
{

    KJB_image* input_ip  = NULL;
    KJB_image* output_ip = NULL;

    Int_matrix* input_mp  = NULL;
    Int_matrix* output_mp = NULL;

    KJB_region_list* region_list_rlp  = NULL;
    Segmentation_t3* segmentation_ptr = NULL;

    int num_rows, num_cols;
    int num_connected_regions;
    int max_region_label;
    int i, j;

    kjb_init();   /* Best to do this if using KJB library. */

    /*
     * Kobus. Added this so I can test the testing.
     * Not ready to do anything in batch mode. 
    */
    if (! is_interactive())
    {
        return EXIT_SUCCESS;
    }
    
    if (argc != 3)
    {
        set_error("Insufficient number of arguments:\nUsage: seg_connected_components <input_image> <output_image>");
        kjb_print_error();
        return EXIT_FAILURE;
    }
 
    EPETE(kjb_read_image_2(&input_ip, argv[1]));

    num_rows = input_ip->num_rows;
    num_cols = input_ip->num_cols;
    
    EPETE(get_zero_int_matrix(&input_mp, num_rows, num_cols));
    
    for (i = 0; i < num_rows; i++)
    {
        for (j = 0; j < num_cols; j++)
        {
            float r = input_ip->pixels[i][j].r;
            float g = input_ip->pixels[i][j].g;
            float b = input_ip->pixels[i][j].b;

            input_mp->elements[i][j] = (int) (r+g+b/3.0);
        }
    }
    
    /* OLD CODE.
    regions_rp = (KJB_region*) KJB_MALLOC(num_rows*num_cols*sizeof(KJB_region));
    */

    /*
    EPETE( label_four_connected_regions(&output_mp,
                                        &region_list_rlp,
                                        &segmentation_ptr,
                                        &num_connected_regions,
                                        &max_region_label,
                                        input_ip,
                                        input_mp,
                                        1) );
    */

    /* Four-connected components */
    /*
    EPETE( label_four_connected_regions(&output_mp,
                                        NULL,
                                        NULL,
                                        &num_connected_regions,
                                        &max_region_label,
                                        NULL,
                                        input_mp,
                                        1) );
    */                                        

    /* Eight-connected components */
    EPETE( label_eight_connected_regions(&output_mp,
                                         NULL,
                                         NULL,
                                         &num_connected_regions,
                                         &max_region_label,
                                         NULL,
                                         input_mp,
                                         1) );

    /* Only for allocating output_ip. */
    EPETE(kjb_copy_image(&output_ip, input_ip));
    
    for (i = 0; i < num_rows; i++)
    {
        for (j = 0; j < num_cols; j++)
        {
            float gray_value = (255.0f / num_connected_regions) * output_mp->elements[i][j]; 
            
            output_ip->pixels[i][j].r = gray_value;
            output_ip->pixels[i][j].g = gray_value;
            output_ip->pixels[i][j].b = gray_value;
        }
    }

    EPETE(kjb_write_image(output_ip, argv[2]));

    kjb_free_image(input_ip);
    kjb_free_image(output_ip);
    free_int_matrix(input_mp);
    free_int_matrix(output_mp);
    free_region_list(region_list_rlp);
    t3_free_segmentation(segmentation_ptr);
    
    kjb_cleanup(); /* Almost never needed, but doing it twice is OK. */

    return EXIT_SUCCESS; 
} 

/*  /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\ /\   */

